@@ -5,15 +5,18 @@ from typing import Optional, Union, MutableMapping
 
 from copy import copy
 from collections import ChainMap
+import itertools
 import xml.etree.ElementTree as ET
 
 from ziafont import Font
 from ziafont.fonttypes import BBox
 from ziafont.glyph import SimpleGlyph
 
+from .drawable import Drawable
 from .styles import styledstr
 from .zmath import MathFont
 from . import operators
+from . import drawable
 
 
 DEBUG = False       # Debug mode, draws bboxes
@@ -24,8 +27,10 @@ def getstyle(element: ET.Element) -> dict:
     ''' Get style arguments based on "mathvariant" in mathml '''
     variant = element.attrib.get('mathvariant', '')
     styleargs: dict[str, Union[str, bool]] = {}
-    if 'italic' in variant:
+    if 'italic' in variant and 'normal' not in variant:
         styleargs['italic'] = True
+    if 'normal' in variant:
+        styleargs['normal'] = True
     if 'bold' in variant:
         styleargs['bold'] = True
     if 'double' in variant:
@@ -40,6 +45,10 @@ def getstyle(element: ET.Element) -> dict:
         styleargs['style'] = 'fraktur'
     if 'displaystyle' in element.attrib:
         styleargs['displaystyle'] = element.attrib['displaystyle']
+    if 'mathcolor' in element.attrib:
+        styleargs['mathcolor'] = element.attrib['mathcolor']
+    if 'mathbackground' in element.attrib:
+        styleargs['mathbackground'] = element.attrib['mathbackground']
     return styleargs
 
 
@@ -68,6 +77,8 @@ def makenode(element: ET.Element, size: float,
             'mroot': Mroot,
             'mtext': Mtext,
             'mspace': Mspace,
+            'mfenced': Mfenced,
+            'menclose': Menclose,
             'mpadded': Mpadded,
             'mphantom': Mphantom,
             'mtable': Mtable,
@@ -121,7 +132,7 @@ def getelementtext(element: ET.Element) -> str:
     return txt
 
 
-class Mnode:
+class Mnode(drawable.Drawable):
     ''' Math Drawing Node
 
         Args:
@@ -134,16 +145,16 @@ class Mnode:
         self.font: MathFont = parent.font
         self.size = size
         self.parent = parent
-        self.style: MutableMapping[str, Union[str, bool]] = ChainMap(copy(parent.style), getstyle(self.element))
+        self.style: MutableMapping[str, Union[str, bool]] = ChainMap(getstyle(self.element), copy(parent.style))
         self.emscale = size / self.font.info.layout.unitsperem
-        self.nodes: list[Mnode] = []
+        self.nodes: list[drawable.Drawable] = []
         self.nodexy: list[tuple[float, float]] = []
 
     def _setup(self, **kwargs) -> None:
         ''' Calculate node position assuming this node is at 0, 0. Also set bbox. '''
         self.bbox = BBox(0, 0, 0, 0)
 
-    def leftsibling(self) -> Optional['Mnode']:
+    def leftsibling(self) -> Optional[drawable.Drawable]:
         ''' Left node sibling. The one that was just placed. '''
         try:
             return self.parent.nodes[-1]
@@ -180,99 +191,20 @@ class Mnode:
             rect.attrib['stroke'] = 'blue'
             rect.attrib['stroke-width'] = '0.2'
 
+        if 'mathbackground' in self.style:
+            rect = ET.SubElement(svg, 'rect')
+            rect.set('x', str(x))
+            rect.set('y', str(y - self.bbox.ymax))
+            rect.set('width', str((self.bbox.xmax - self.bbox.xmin)))
+            rect.set('height', str((self.bbox.ymax - self.bbox.ymin)))
+            rect.set('fill', self.style['mathbackground'])  # type: ignore
+            
         xi = yi = 0.
         for (xi, yi), node in zip(self.nodexy, self.nodes):
             node.draw(x+xi, y+yi, svg)
         return x+xi, y+yi
 
 
-class MGlyph(Mnode):
-    ''' A single glyph node (not <mglyph>, but used by other nodes) '''
-    def __init__(self, glyph: SimpleGlyph, char: str, size: float, emscale: float, **kwargs):
-        self.glyph = glyph
-        self.char = char
-        self.size = size
-        self.emscale = emscale
-        self.phantom = kwargs.get('phantom', False)
-        self._setup()
-
-    def _setup(self, **kwargs) -> None:
-        ''' Place the glyphs with 0, 0 positions '''
-        self.bbox = BBox(
-            self.glyph.path.bbox.xmin * self.emscale,
-            self.glyph.path.bbox.xmax * self.emscale,
-            self.glyph.path.bbox.ymin * self.emscale,
-            self.glyph.path.bbox.ymax * self.emscale)
-
-    def firstglyph(self) -> Optional[SimpleGlyph]:
-        ''' Get the first glyph in this node '''
-        return self.glyph
-
-    def lastglyph(self) -> Optional[SimpleGlyph]:
-        ''' Get the last glyph in this node '''
-        return self.glyph
-
-    def lastchar(self) -> Optional[str]:
-        ''' Get the last character in this node '''
-        return self.char
-
-    def draw(self, x: float, y: float, svg: ET.Element) -> tuple[float, float]:
-        ''' Draw the node on the SVG
-
-            Args:
-                x: Horizontal position in SVG coordinates
-                y: Vertical position in SVG coordinates
-                svg: SVG drawing as XML
-        '''
-        symbols = svg.findall('symbol')
-        symids = [sym.attrib.get('id') for sym in symbols]
-        if self.glyph.id not in symids and self.glyph.font.svg2:
-            svg.append(self.glyph.svgsymbol())
-        if not self.phantom:
-            svg.append(self.glyph.place(x, y, self.size))
-        x += self.glyph.advance() * self.emscale
-        return x, y
-
-
-class MHLine(Mnode):
-    ''' SVG Horizontal Line. Used by mfrac and msqrt. '''
-    def __init__(self, length: float, lw: float, **kwargs):
-        self.length = length
-        self.lw = lw
-        self.phantom = kwargs.get('phantom', False)
-        self.bbox = BBox(0, lw/2, self.length, self.lw)
-
-    def firstglyph(self) -> Optional[SimpleGlyph]:
-        ''' Get the first glyph in this node '''
-        return None
-
-    def lastglyph(self) -> Optional[SimpleGlyph]:
-        ''' Get the last glyph in this node '''
-        return None
-
-    def lastchar(self) -> Optional[str]:
-        ''' Get the last character in this node '''
-        return None
-
-    def draw(self, x: float, y: float, svg: ET.Element) -> tuple[float, float]:
-        ''' Draw the node on the SVG
-
-            Args:
-                x: Horizontal position in SVG coordinates
-                y: Vertical position in SVG coordinates
-                svg: SVG drawing as XML
-        '''
-        if not self.phantom:
-            # Use rectangle so it can change color with 'fill' attribute
-            # and not mess up glyphs with 'stroke' attribute
-            bar = ET.SubElement(svg, 'rect')
-            bar.attrib['x'] = str(x)
-            bar.attrib['y'] = str(y)
-            bar.attrib['width'] = str(self.length)
-            bar.attrib['height'] = str(self.lw)
-        return x+self.length, y
-
-
 class Midentifier(Mnode):
     ''' Identifier node <mi> '''
     def __init__(self, element: ET.Element, size: float, parent: Mnode, **kwargs):
@@ -280,7 +212,7 @@ class Midentifier(Mnode):
 
         # Identifiers are italic unless longer than one character
         text = getelementtext(self.element)
-        if len(text) == 1 and 'italic' not in self.style:
+        if len(text) == 1 and 'italic' not in self.style and 'normal' not in self.style:
             self.style['italic'] = True
         if len(text) > 1:
             text += ' '
@@ -293,7 +225,7 @@ class Midentifier(Mnode):
         x = 0
         for char in self.string:
             glyph = self.font.glyph(char)
-            self.nodes.append(MGlyph(glyph, char, self.size, self.emscale, **kwargs))
+            self.nodes.append(drawable.Glyph(glyph, char, self.size, self.emscale, self.style, **kwargs))
             self.nodexy.append((x, 0))
             x += glyph.advance() * self.emscale
             ymin = min(ymin, glyph.path.bbox.ymin * self.emscale)
@@ -379,65 +311,104 @@ class Mrow(Mnode):
 
     def _setup(self, **kwargs) -> None:
         kwargs = copy(kwargs)
+        node: Mnode
+        self.nodes = []
+
+        # Break mrow into lines - to handle mspace linebreak (// in latex)
+        lines = []
+        line: list[ET.Element] = []
         for i, child in enumerate(self.element):
             if child.tag == 'mo':
                 infer_opform(i, child, self)
-
-        self.nodes = []
-        x = 0
-        ymax = -9999
-        ymin = 9999
-        i = 0
-        node: Mnode
-        while i < len(self.element):
-            child = self.element[i]
-            text = getelementtext(child)
-            if child.tag == 'mo':
-                if (isstretchy(text, self.font) and
-                    child.attrib.get('form') == 'prefix' and
-                    child.attrib.get('stretchy') != 'false'):
-                    fencekwargs = copy(kwargs)
-                    j = 0
-                    for j in range(i+1, len(self.element)):
-                        if self.element[j].tag == 'mo' and self.element[j].attrib.get('form') == 'postfix':
-                            children = self.element[i+1: j]
-                            fencekwargs['open'] = getelementtext(child)
-                            fencekwargs['close'] = getelementtext(self.element[j])
-                            break
-                    else:  # No postfix closing fence. Enclose remainder of row.
-                        children = self.element[i+1:]
-                        fencekwargs['open'] = getelementtext(child)
-                        fencekwargs['close'] = None
-                    fenced = ET.Element('fenceop')
-                    fenced.attrib.update(child.attrib)
-                    frow = ET.SubElement(fenced, 'mrow')
-                    frow.extend(children)
-                    node = Mopfence(fenced, self.size, parent=self, **fencekwargs)
-                    i = j + 1
-
-                else:
-                    if text == '':
-                        i += 1
-                        continue  # InvisibleTimes, etc.
-                    node = Moperator(child, self.size, parent=self, **kwargs)
-                    i += 1
+            if child.tag == 'mspace' and child.attrib.get('linebreak', None) == 'newline':
+                lines.append(line)
+                line = []
             else:
-                node = makenode(child, self.size, parent=self, **kwargs)
-                i += 1
-            self.nodes.append(node)
-            self.nodexy.append((x, 0))
-            x += node.bbox.xmax
-            ymax = max(ymax, node.bbox.ymax)
-            ymin = min(ymin, node.bbox.ymin)
-        self.bbox = BBox(0, x, ymin, ymax)
+                line.append(child)
+        lines.append(line)
+
+        if len(lines) > 1:
+            # Multiline - process each line as an mrow so we can get its bounding box
+            for i, line in enumerate(lines):
+                mrowelm = ET.Element('mrow')
+                mrowelm.extend(line)
+                node = Mrow(mrowelm, self.size, parent=self)
+                self.nodes.append(node)
+
+            y = 0
+            for i, node in enumerate(self.nodes):  # type: ignore
+                if i > 0:
+                    y += (node.bbox.ymax - self.nodes[i-1].bbox.ymin + self.font.math.consts.mathLeading*self.emscale*2)  # type: ignore
+                self.nodexy.append((0, y))
+            xmax = max([n.bbox.xmax for n in self.nodes])    # type: ignore
+            ymin = -y+self.nodes[-1].bbox.ymin    # type: ignore
+            ymax = self.nodes[0].bbox.ymax    # type: ignore
+            self.bbox = BBox(0, xmax, ymin, ymax)
+        else:
+            # Single line
+            ymax = -9999
+            ymin = 9999
+            i = 0
+            x = 0
+            while i < len(line):
+                child = line[i]
+                text = getelementtext(child)
+                if child.tag == 'mo':
+                    if (isstretchy(text, self.font) and
+                        child.attrib.get('form') == 'prefix' and
+                        child.attrib.get('stretchy') != 'false'):
+                        fencekwargs = copy(kwargs)
+                        j = 0
+                        for j in range(i+1, len(self.element)):
+                            if self.element[j].tag == 'mo' and self.element[j].attrib.get('form') == 'postfix':
+                                children = self.element[i+1: j]
+                                fencekwargs['open'] = getelementtext(child)
+                                fencekwargs['close'] = getelementtext(self.element[j])
+                                break
+                        else:  # No postfix closing fence. Enclose remainder of row.
+                            children = self.element[i+1:]
+                            fencekwargs['open'] = getelementtext(child)
+                            fencekwargs['close'] = None
+                        fencekwargs['separators'] = ''
+                        fenced = ET.Element('mfenced')
+                        fenced.attrib.update(child.attrib)
+                        fenced.attrib.update(fencekwargs)
+                        if len(children) > 0:
+                            frow = ET.SubElement(fenced, 'mrow')
+                            frow.extend(children)
+                        node = Mfenced(fenced, self.size, parent=self, **kwargs)
+                        i = j + 1
+
+                    else:
+                        if text == '':
+                            i += 1
+                            continue  # InvisibleTimes, etc.
+                        node = Moperator(child, self.size, parent=self, **kwargs)
+                        i += 1
+                else:
+                    node = makenode(child, self.size, parent=self, **kwargs)
+                    i += 1
+
+                self.nodes.append(node)
+                self.nodexy.append((x, 0))
+                x += node.bbox.xmax
+                ymax = max(ymax, node.bbox.ymax)
+                ymin = min(ymin, node.bbox.ymin)
+            self.bbox = BBox(0, x, ymin, ymax)
 
     def firstglyph(self) -> Optional[SimpleGlyph]:
         ''' Get the first glyph in this node '''
-        return self.nodes[0].firstglyph()
+        try:
+            return self.nodes[0].firstglyph()
+        except IndexError:
+            return None
 
     def lastglyph(self) -> Optional[SimpleGlyph]:
         ''' Get the last glyph in this node '''
-        return self.nodes[-1].lastglyph()
+        try:
+            return self.nodes[-1].lastglyph()
+        except IndexError:
+            return None
 
     def lastchar(self) -> Optional[str]:
         ''' Get the last character in this node '''
@@ -446,64 +417,99 @@ class Mrow(Mnode):
         except IndexError:
             return None
 
-
-class Mopfence(Mnode):
-    ''' Fenced operator. MathML doesn't put items inside fence () in xml subelement
-        which means it's not easy to calculate the stretch size of the fence glyphs.
-        This class wraps the content of the fence and handles both opening and closing
-        operators.
+        
+class Mfenced(Mnode):
+    ''' Mfence element. Puts contents in parenthesis or other fence glyphs, with
+        optional separators between components.
     '''
     def __init__(self, element: ET.Element, size: float, parent: Mnode, **kwargs):
         super().__init__(element, size, parent, **kwargs)
-        assert len(element) == 1  # Should contain 1 mrow with contents of Opfence
-        self.openchr = kwargs.get('open')
-        self.closechr = kwargs.get('close')
+        self.openchr = element.attrib.get('open', '(')
+        self.closechr = element.attrib.get('close', ')')
+        self.separators = element.attrib.get('separators', ',').replace(' ', '')
         self._setup(**kwargs)
 
     def _setup(self, **kwargs) -> None:
-        mrow = Mrow(self.element[0], self.size, parent=self)
-        height = mrow.bbox.ymax - mrow.bbox.ymin
+        separator_elms = [ET.fromstring(f'<mo>{k}</mo>') for k in self.separators]
+        fencedelms: Union[list[ET.Element], ET.Element] = []
+        # Insert separators
+        if len(self.element) > 1 and len(self.separators) > 0:
+            fencedelms = list(itertools.chain.from_iterable(
+                itertools.zip_longest(self.element, separator_elms, fillvalue=separator_elms[-1])))   # flatten
+            fencedelms = fencedelms[:len(self.element)*2-1]  # Remove any separators at end
+        else:
+            # Single element in fence, no separators
+            fencedelms = self.element
+
+        mrowelm = ET.Element('mrow')
+        mrowelm.extend(fencedelms)
+        mrow = Mrow(mrowelm, self.size, parent=self)
+        if len(mrow.nodes) == 0:
+            # Opening fence with nothing in it
+            openglyph = self.font.glyph(self.openchr)
+            mglyph = drawable.Glyph(openglyph, self.openchr, self.size, self.emscale, self.style, **kwargs)
+            height = mglyph.bbox.ymax - mglyph.bbox.ymin
+            fencebbox = mglyph.bbox
+        else:
+            height = mrow.bbox.ymax - mrow.bbox.ymin
+            fencebbox = mrow.bbox
         self.nodes = []
         x = 0
-        if height > self.size * 1.5:
-            # Tall enclosures (like tables) should be centered vertically
+        if len(mrow.nodes) and isinstance(mrow.nodes[0], Mtable):
+            # Tables should be centered vertically
             rowbaseline = -self.font.math.consts.axisHeight * self.emscale
         else:
             rowbaseline = 0
-        rowcenter = rowbaseline - mrow.bbox.ymin - (mrow.bbox.ymax - mrow.bbox.ymin)/2
+
+        rowcenter = rowbaseline - fencebbox.ymin - (fencebbox.ymax - fencebbox.ymin)/2
+
+        yglyphmin = 0
+        yglyphmax = 0
 
         if self.openchr:
             openglyph = self.font.glyph(self.openchr)
             openglyph = self.font.math.variant(openglyph.index, height/self.emscale, vert=True)
-            mglyph = MGlyph(openglyph, self.openchr, self.size, self.emscale, **kwargs)
+            mglyph = drawable.Glyph(openglyph, self.openchr, self.size, self.emscale, self.style, **kwargs)
             yofst = rowcenter + mglyph.bbox.ymin + (mglyph.bbox.ymax - mglyph.bbox.ymin)/2
 
             self.nodes.append(mglyph)
             self.nodexy.append((x, yofst))
             x += openglyph.advance() * self.emscale
+            yglyphmin = min(yofst+mglyph.bbox.ymin, yglyphmin)
+            yglyphmax = max(yofst+mglyph.bbox.ymax, yglyphmax)
 
-        self.nodes.append(mrow)
-        self.nodexy.append((x, rowbaseline))
-        x += mrow.bbox.xmax
+        if len(fencedelms) > 0:
+            self.nodes.append(mrow)
+            self.nodexy.append((x, rowbaseline))
+            x += fencebbox.xmax
 
         if self.closechr:
             closeglyph = self.font.glyph(self.closechr)
             closeglyph = self.font.math.variant(closeglyph.index, height/self.emscale, vert=True)
-
+            mglyph = drawable.Glyph(closeglyph, self.closechr, self.size, self.emscale, self.style, **kwargs)
             yofst = rowcenter + mglyph.bbox.ymin + (mglyph.bbox.ymax - mglyph.bbox.ymin)/2
-            self.nodes.append(MGlyph(closeglyph, self.closechr, self.size, self.emscale, **kwargs))
+
+            self.nodes.append(mglyph)
             self.nodexy.append((x, yofst))
             x += closeglyph.advance() * self.emscale
+            yglyphmin = min(yofst+mglyph.bbox.ymin, yglyphmin)
+            yglyphmax = max(yofst+mglyph.bbox.ymax, yglyphmax)
 
-        self.bbox = BBox(0, x, -rowbaseline+mrow.bbox.ymin, -rowbaseline+mrow.bbox.ymax)
+        self.bbox = BBox(0, x, min(yglyphmin, -rowbaseline+fencebbox.ymin), max(yglyphmax, -rowbaseline+fencebbox.ymax))
 
     def firstglyph(self) -> Optional[SimpleGlyph]:
         ''' Get the first glyph in this node '''
-        return self.nodes[0].firstglyph()
+        try:
+            return self.nodes[0].firstglyph()
+        except IndexError:
+            return None
 
     def lastglyph(self) -> Optional[SimpleGlyph]:
         ''' Get the last glyph in this node '''
-        return self.nodes[-1].firstglyph()
+        try:
+            return self.nodes[-1].lastglyph()
+        except IndexError:
+            return None
 
     def lastchar(self) -> Optional[str]:
         ''' Get the last character in this node '''
@@ -551,7 +557,7 @@ class Moperator(Mnumber):
             if self.width:
                 glyph = self.font.math.variant(glyph.index, self.width / self.emscale, vert=False)
 
-            self.nodes.append(MGlyph(glyph, char, self.size, self.emscale, **kwargs))
+            self.nodes.append(drawable.Glyph(glyph, char, self.size, self.emscale, self.style, **kwargs))
             self.nodexy.append((x, 0))
             x += glyph.advance() * self.emscale
             ymin = min(ymin, glyph.path.bbox.ymin * self.emscale)
@@ -642,19 +648,31 @@ class Msup(Mnode):
         supx, supy, xadv = place_super(self.base, self.superscript, self.font, self.emscale)
         self.nodes.append(self.superscript)
         self.nodexy.append((x+supx, supy))
-        xmin = self.base.bbox.xmin
-        xmax = x + xadv
-        ymin = self.base.bbox.ymin
-        ymax = max(self.base.bbox.ymax, -supy + self.superscript.bbox.ymax)
+        if self.base.bbox.ymax > self.base.bbox.ymin:
+            xmin = self.base.bbox.xmin
+            xmax = x + xadv
+            ymin = self.base.bbox.ymin
+            ymax = max(self.base.bbox.ymax, -supy + self.superscript.bbox.ymax)
+        else:
+            xmin = 0
+            ymin = -supy
+            xmax = x + xadv
+            ymax = -supy + self.superscript.bbox.ymax
         self.bbox = BBox(xmin, xmax, ymin, ymax)
 
     def firstglyph(self) -> Optional[SimpleGlyph]:
         ''' Get the first glyph in this node '''
-        return self.nodes[0].firstglyph()
+        try:
+            return self.nodes[0].firstglyph()
+        except IndexError:
+            return None
 
     def lastglyph(self) -> Optional[SimpleGlyph]:
         ''' Get the last glyph in this node '''
-        return self.nodes[-1].lastglyph()
+        try:
+            return self.nodes[-1].lastglyph()
+        except IndexError:
+            return None
 
     def lastchar(self) -> Optional[str]:
         ''' Get the last character in this node '''
@@ -692,11 +710,17 @@ class Msub(Mnode):
 
     def firstglyph(self) -> Optional[SimpleGlyph]:
         ''' Get the first glyph in this node '''
-        return self.nodes[0].firstglyph()
+        try:
+            return self.nodes[0].firstglyph()
+        except IndexError:
+            return None
 
     def lastglyph(self) -> Optional[SimpleGlyph]:
         ''' Get the last glyph in this node '''
-        return self.nodes[-1].lastglyph()
+        try:
+            return self.nodes[-1].lastglyph()
+        except IndexError:
+            return None
 
     def lastchar(self) -> Optional[str]:
         ''' Get the last character in this node '''
@@ -746,11 +770,17 @@ class Msubsup(Mnode):
 
     def firstglyph(self) -> Optional[SimpleGlyph]:
         ''' Get the first glyph in this node '''
-        return self.nodes[0].firstglyph()
+        try:
+            return self.nodes[0].firstglyph()
+        except IndexError:
+            return None
 
     def lastglyph(self) -> Optional[SimpleGlyph]:
         ''' Get the last glyph in this node '''
-        return self.nodes[-1].lastglyph()
+        try:
+            return self.nodes[-1].lastglyph()
+        except IndexError:
+            return None
 
     def lastchar(self) -> Optional[str]:
         ''' Get the last character in this node '''
@@ -956,7 +986,7 @@ class Mfrac(Mnode):
                              'thick': linethick * 2}.get(lt, linethick)
 
         bary = -self.font.math.consts.axisHeight*self.emscale
-        self.nodes.append(MHLine(width, linethick, **kwargs))
+        self.nodes.append(drawable.HLine(width, linethick, style=self.style, **kwargs))
         self.nodexy.append((x, bary))
 
         # Calculate/cache bounding box
@@ -982,7 +1012,7 @@ class Mroot(Mnode):
 
         # Get the right glyph to fit the contents
         rglyph = self.font.math.variant(self.font.glyphindex('√'), height/self.emscale, vert=True)
-        rootnode = MGlyph(rglyph, '√', self.size, self.emscale, **kwargs)
+        rootnode = drawable.Glyph(rglyph, '√', self.size, self.emscale, self.style, **kwargs)
 
         # Shift radical up/down to ensure minimum gap between top of text and overbar
         # Keep contents at the same height.
@@ -1021,7 +1051,7 @@ class Mroot(Mnode):
             if italicx:
                 width += italicx * self.emscale
 
-        self.nodes.append(MHLine(width, self.font.math.consts.radicalRuleThickness * self.emscale, **kwargs))
+        self.nodes.append(drawable.HLine(width, self.font.math.consts.radicalRuleThickness * self.emscale, style=self.style, **kwargs))
         self.nodexy.append((x, yrad-rglyph.path.bbox.ymax * self.emscale))
         xmin = rglyph.path.bbox.xmin * self.emscale
         xmax = x + width
@@ -1047,6 +1077,87 @@ class Msqrt(Mroot):
         self._setup(**kwargs)
 
 
+class Menclose(Mnode):
+    ''' Enclosure '''
+    def __init__(self, element: ET.Element, size: float, parent: Mnode, **kwargs):
+        super().__init__(element, size, parent, **kwargs)
+        if len(self.element) > 1:
+            row = ET.Element('mrow')
+            row.extend(list(self.element))
+            self.base = makenode(row, size, parent=self, **kwargs)
+        else:
+            self.base = makenode(self.element[0], size, parent=self, **kwargs)
+        self.notation = element.attrib.get('notation', 'box').split()
+        self._setup(**kwargs)
+
+    def _setup(self, **kwargs) -> None:
+        pad = self.font.math.consts.radicalRuleThickness * self.emscale * 2
+        height = self.base.bbox.ymax - self.base.bbox.ymin + pad * 2
+        width = self.base.bbox.xmax - self.base.bbox.xmin + pad * 2
+        lw = self.font.math.consts.radicalRuleThickness * self.emscale
+        basex = pad
+        xarrow = 0
+        yarrow = 0
+
+        if 'box' in self.notation:
+            self.nodes.append(drawable.Box(width, height, lw, style=self.style, **kwargs))
+            self.nodexy.append((0, -self.base.bbox.ymax+height-pad))
+        if 'circle' in self.notation:
+            self.nodes.append(drawable.Ellipse(width, height, lw, style=self.style, **kwargs))
+            self.nodexy.append((0, -self.base.bbox.ymax+height-pad))
+        if 'roundedbox' in self.notation:
+            self.nodes.append(drawable.Box(width, height, lw, style=self.style, cornerradius=lw*4, **kwargs))
+            self.nodexy.append((0, -self.base.bbox.ymax+height-pad))
+            
+        if ('top' in self.notation or
+            'longdiv' in self.notation or
+            'actuarial' in self.notation):
+            self.nodes.append(drawable.HLine(width, lw, style=self.style, **kwargs))
+            self.nodexy.append((0, -self.base.bbox.ymax-pad))
+        if ('bottom' in self.notation or
+            'madruwb' in self.notation or
+            'phasorangle' in self.notation):
+            self.nodes.append(drawable.HLine(width, lw, style=self.style, **kwargs))
+            self.nodexy.append((0, -self.base.bbox.ymin+pad))
+        if ('right' in self.notation or
+            'madruwb' in self.notation or
+            'actuarial' in self.notation):
+            self.nodes.append(drawable.VLine(height, lw, style=self.style, **kwargs))
+            self.nodexy.append((self.base.bbox.xmax+pad*2, -self.base.bbox.ymax-pad))
+        if ('left' in self.notation or
+            'longdiv' in self.notation):
+            self.nodes.append(drawable.VLine(height, lw, style=self.style, **kwargs))
+            self.nodexy.append((0, -self.base.bbox.ymax-pad))
+        if 'verticalstrike' in self.notation:
+            self.nodes.append(drawable.VLine(height, lw, style=self.style, **kwargs))
+            self.nodexy.append((width/2, -self.base.bbox.ymax-pad))
+        if 'horizontalstrike' in self.notation:
+            self.nodes.append(drawable.HLine(width, lw, style=self.style, **kwargs))
+            self.nodexy.append((0, -self.base.bbox.ymin-height/2))
+            
+        if 'updiagonalstrike' in self.notation:
+            self.nodes.append(drawable.Diagonal(width, -height, lw, style=self.style, **kwargs))
+            self.nodexy.append((0, -self.base.bbox.ymin-height+pad))
+        if 'downdiagonalstrike' in self.notation:
+            self.nodes.append(drawable.Diagonal(width, height, lw, style=self.style, **kwargs))
+            self.nodexy.append((0, -self.base.bbox.ymin+pad))
+        if 'phasorangle' in self.notation:
+            self.nodes.append(drawable.Diagonal(height/3, -height, lw, style=self.style, **kwargs))
+            self.nodexy.append((0, -self.base.bbox.ymin-height+pad))
+            basex += height/4  # Shift base right a bit so it fits under angle
+
+        if 'updiagonalarrow' in self.notation:
+            self.nodes.append(drawable.Diagonal(width, -height, lw, style=self.style, arrow=True, **kwargs))
+            self.nodexy.append((0, -self.base.bbox.ymin-height+pad))
+            xarrow = self.nodes[-1].arroww  # type: ignore
+            yarrow = self.nodes[-1].arrowh  # type: ignore
+
+        self.nodes.append(self.base)
+        self.nodexy.append((basex, 0))
+
+        self.bbox = BBox(0, basex+width+xarrow, self.base.bbox.ymin-pad, height-pad+yarrow)
+
+
 class Mspace(Mnode):
     ''' Blank space '''
     def __init__(self, element: ET.Element, size: float, parent: Mnode, **kwargs):
